@@ -263,17 +263,6 @@ class WAS (wsgiappservice.WAS):
     def render (self, template_file, _do_not_use_this_variable_name_ = {}, **karg):
         return self.app.render (self, template_file, _do_not_use_this_variable_name_, **karg)
 
-    def API (self, *args, **karg):
-        api = self.response.API (*args, **karg)
-        try: api.set_json_encoder (self.app.config.get ("JSON_ENCODER"), self.app.config.get ('PRETTY_JSON', False))
-        except AttributeError: pass
-        return api
-
-    def render_or_API (self, template_file, _do_not_use_this_variable_name_ = {}, **karg):
-        if self.request.acceptable ('application/json'):
-            return self.API (None, _do_not_use_this_variable_name_, **karg)
-        return self.render (template_file, _do_not_use_this_variable_name_ = {}, **karg)
-
     def Fault (self, *args, **karg):
         fault = self.response.Fault (*args, **karg)
         try: fault.set_json_encoder (self.app.config.get ("JSON_ENCODER"))
@@ -296,56 +285,6 @@ class WAS (wsgiappservice.WAS):
         _was, args, kargs = self._insert_cloned_was (args, kargs)
         return self.executors.create_thread (_was.ID, target, *args, **kargs).then ()
 
-    def Map (self, *commits, **corequests):
-        def mapped_api (was, tasks, keys, status):
-            results = tasks.fetch ()
-            if tasks.count () == 1 and not isinstance (results, (list, tuple)):
-                results = [results]
-
-            data = {}
-            for idx, key in enumerate (keys):
-                if key is None:
-                    continue
-
-                keydata = results [idx]
-                method, filed = 'fetch', None
-                option = key.split ('__')
-                if len (option) == 3:
-                    key, method, field = option
-                    if method  != 'one':
-                        raise RuntimeError ('must be `one` for specifying field')
-                elif len (option) == 2:
-                    key, method = option
-                    if method not in ('one', 'fetch'):
-                        field, method = method, 'one'
-
-                if method == 'one':
-                    if len (keydata) == 0:
-                        raise was.Error ('410 Partial Not Found')
-                    if len (keydata) > 1:
-                        raise was.Error ('409 Conflict')
-                    keydata = keydata [0]
-                    if field:
-                        keydata = keydata [field]
-
-                data [key] = keydata
-            return was.API (status, data)
-
-        status = "200 OK"
-        if commits and isinstance (commits [0], str):
-            status, commits = commits [0], commits [1:]
-
-        keys = []
-        ts = []
-        for k, v in corequests.items ():
-            keys.append (k)
-            ts.append (v)
-        for k in commits:
-            keys.append (None)
-            ts.append (k)
-        tasks = len (ts) == 1 and ts [0] or self.Tasks (*ts)
-        return tasks.then (partial (mapped_api, keys = keys, status = status))
-
     def File (self, *args, **karg):
         return self.response.File (*args, **karg)
 
@@ -367,3 +306,52 @@ class WAS (wsgiappservice.WAS):
         headers ["Accept"] = "*/*"
         return self.get ("{}/{}".format (alias, path), headers = headers, timeout = timeout).then (respond)
     proxypass = ProxyPass
+
+    # API responses ------------------------------------
+    def API (self, *args, **karg):
+        api = self.response.API (*args, **karg)
+        try: api.set_json_encoder (self.app.config.get ("JSON_ENCODER"), self.app.config.get ('PRETTY_JSON', False))
+        except AttributeError: pass
+        return api
+
+    def render_or_API (self, template_file, _do_not_use_this_variable_name_ = {}, **karg):
+        if self.request.acceptable ('application/json'):
+            return self.API (None, _do_not_use_this_variable_name_, **karg)
+        return self.render (template_file, _do_not_use_this_variable_name_ = {}, **karg)
+
+    # Map responses ------------------------------------
+    def _mapped_api_callback (self, was, tasks, status, template_file = None):
+        data = tasks.dict ()
+        if template_file:
+            return was.render (template_file, data)
+        else:
+            return was.API (status, data)
+
+    def Mapped (self, *tasks):
+        try:
+            status, tasks = tasks
+        except ValueError:
+            status, tasks = "200 OK", tasks [0]
+        respfunc = partial (self._mapped_api_callback, status = status)
+        return tasks.then (respfunc)
+
+    def render_or_Mapped (self, template_file, tasks):
+        if self.request.acceptable ('application/json'):
+            return self.Mapped (tasks)
+        respfunc = partial (self._mapped_api_callback, status = '200 OK', template_file = template_file)
+        return tasks.then (respfunc)
+
+    def Map (self, *commits, **corequests):
+        status = "200 OK"
+        if commits and isinstance (commits [0], str):
+            status, commits = commits [0], commits [1:]
+        tasks = self.Tasks (*commits, **corequests)
+        respfunc = partial (self._mapped_api_callback, status = status)
+        return tasks.then (respfunc)
+
+    def render_or_Map (self, template_file, *commits, **corequests):
+        if self.request.acceptable ('application/json'):
+            return self.Map (*commits, **corequests)
+        tasks = self.Tasks (*commits, **corequests)
+        respfunc = partial (self._mapped_api_callback, status = '200 OK', template_file = template_file)
+        return tasks.then (respfunc)
