@@ -275,21 +275,26 @@ class Executor:
 
 		except MemoryError:
 			raise
+
 		except AssertionError as e:
 			self.rollback ()
 			content = [request.response.with_explain ('400 Not My Fault', *e.args [:1])]
+
 		except HTTPError as e:
 			self.rollback ()
 			content = [request.response.with_explain (e.status, e.explain or (self.was.app.debug and e.exc_info), e.errno)]
+
 		except:
 			self.was.traceback ()
 			self.rollback ()
 			content = [request.response ("502 Bad Gateway", exc_info = self.was.app.debug and sys.exc_info () or None)]
+
 		else:
 			self.commit ()
 
-		if current_app.debug and current_app.expose_spec and request.channel.addr [0].startswith ("127.0.0.") and isinstance (content [0], API):
-			content [0].set_spec (current_app)
+		if current_app.expose_spec and current_app.debug and request.channel:
+			if request.channel.addr [0].startswith ("127.0.0.") and isinstance (content [0], API):
+				content [0].set_spec (current_app)
 
 		# clean was
 		del self.was.subapp
